var pow = Math.pow;

var isNaN = Number.isNaN,
    isFinite = Number.isFinite;

function createRounder(options) {
  var decOp = options.dec,
      fixed = options.fixed;
  var dec = decOp != null ? +decOp : 2;

  if (isNaN(dec) || !isFinite(dec) || dec < 0) {
    throw new TypeError('invalid "dec" option.');
  }

  if (fixed) {
    return function (num) {
      return num.toFixed(dec);
    };
  }

  var mul = pow(10, dec);
  return function (num) {
    return Math.round(num * mul) / mul;
  };
}

function createUnitFinder(table, unity) {
  return function (value) {
    if (!value) {
      return unity;
    }

    var val = Math.abs(value);
    var last = table.length - 1;

    for (var i = 0; i < last; i++) {
      var obj = table[i];

      if (val >= obj.div) {
        return obj;
      }
    }

    return table[last];
  };
}

function sortFindUnitArray(units) {
  return units.sort(function (a, b) {
    return b.div - a.div;
  });
}

function createLegacyUnitFinder(table) {
  var unity = {
    pre: '',
    div: 1
  };

  if (!table) {
    return createUnitFinder([{
      pre: 'T',
      div: 1e12
    }, {
      pre: 'G',
      div: 1e9
    }, {
      pre: 'M',
      div: 1e6
    }, {
      pre: 'K',
      div: 1e3
    }, unity, {
      pre: 'm',
      div: 1e-3
    }, {
      pre: "\xB5",
      div: 1e-6
    }, {
      pre: 'n',
      div: 1e-9
    }, {
      pre: 'p',
      div: 1e-12
    }, {
      pre: 'f',
      div: 1e-15
    }], unity);
  }

  return createUnitFinder(sortFindUnitArray(table.map(function (_ref) {
    var pre = _ref.pre,
        power = _ref.power;
    return {
      pre: pre,
      div: pow(10, power)
    };
  })), unity);
}

function formatOutput(value, pre, unit) {
  var wholeUnit = "" + pre + unit;
  return "" + value + (wholeUnit ? " " + wholeUnit : '');
}

function isFunction(x) {
  return typeof x === 'function';
}

function createFormatter(options) {
  var op = options || {};
  var unit = op.unit,
      find = op.find,
      round = op.round,
      output = op.output,
      dec = op.dec,
      fixed = op.fixed;
  var getUnit = isFunction(unit) ? unit : function () {
    return unit || '';
  };
  var findUnit = isFunction(find) ? find : Array.isArray(find) ? createUnitFinder(sortFindUnitArray(find), {
    pre: '',
    div: 1
  }) : createLegacyUnitFinder(op.table);
  var roundNum = isFunction(round) ? round : createRounder(typeof round === 'number' ? {
    dec: round
  } : round || {
    dec: dec,
    fixed: fixed
  });
  var fmt = isFunction(output) ? output : formatOutput;
  return function (value) {
    var unitObj = findUnit(value);
    var pre = unitObj.pre;
    var value2 = value / unitObj.div;
    var rounded = roundNum(value2);
    return fmt(rounded, pre, getUnit(value2, rounded, pre));
  };
}

var defaultTable = [{
  pre: 'meg',
  power: 6
}, {
  pre: 'f',
  power: -15
}, {
  pre: 'p',
  power: -12
}, {
  pre: 'n',
  power: -9
}, {
  pre: 'u',
  power: -6
}, {
  pre: 'm',
  power: -3
}, {
  pre: 'k',
  power: 3
}, {
  pre: 'K',
  power: 3
}, {
  pre: 'M',
  power: 6
}, {
  pre: 'G',
  power: 9
}, {
  pre: 'T',
  power: 12
}];

function createMulFinder(table, unitOp) {
  if (!unitOp) {
    return function (unit) {
      for (var i = 0, len = table.length; i < len; i++) {
        var obj = table[i];

        if (unit === obj.pre) {
          return pow(10, obj.power);
        }
      }

      return NaN;
    };
  }

  return function (unit) {
    if (unit === unitOp) {
      return 1;
    }

    for (var i = 0, len = table.length; i < len; i++) {
      var obj = table[i];

      if (unit === obj.pre || unit === "" + obj.pre + unitOp) {
        return pow(10, obj.power);
      }
    }

    return NaN;
  };
}

function createParser(options) {
  var op = options || {};
  var table = op.table || defaultTable;
  var findMul = createMulFinder(table, op.unit);
  return function (input) {
    if (typeof input === 'number') {
      return input;
    }

    var asString = "" + input;
    var asNum = +asString;

    if (!isNaN(asNum)) {
      return asNum;
    }

    var result = /^\s*(-?[.\d]+(?:e[+-]?\d+)?)\s*(\w*)\s*$/.exec(asString);

    if (!result) {
      return NaN;
    }

    var valueAsStr = result[1],
        unit = result[2];
    var valueAsNum = +valueAsStr;

    if (isNaN(valueAsNum)) {
      return NaN;
    }

    var mul = findMul(unit);

    if (isNaN(mul)) {
      return NaN;
    }

    return valueAsNum * mul;
  };
}

function format(value, options) {
  return createFormatter(options)(value);
}

function parse(input, options) {
  return createParser(options)(input);
}

export { createFormatter, createParser, format, parse };
//# sourceMappingURL=gen-unit.mjs.map
